import './patchAnalytics.js';
import initTcfTracking from './tcf.js';
import analytics from './segmentWrapper.js';
import { defaultContextProperties } from './middlewares/source/defaultContextProperties.js';
import { userScreenInfo } from './middlewares/source/userScreenInfo.js';
import { pageReferrer } from './middlewares/source/pageReferrer.js';
import { userTraits } from './middlewares/source/userTraits.js';
import { getUniversalIdAndNotify } from './universalId.js';
import { getConfig, isClient } from './config.js';
/* Initialize TCF Tracking with Segment */

initTcfTracking();
/* Generate UniversalId if available */

try {
  getUniversalIdAndNotify();
} catch (e) {
  console.error("[segment-wrapper] UniversalID couldn't be initialized");
}
/* Initialize middlewares */


var addMiddlewares = function addMiddlewares() {
  window.analytics.addSourceMiddleware(userTraits);
  window.analytics.addSourceMiddleware(defaultContextProperties);
  window.analytics.addSourceMiddleware(userScreenInfo);
  window.analytics.addSourceMiddleware(pageReferrer);
};
/* Initialize Segment on Client */


if (isClient && window.analytics) {
  /* Initialize Google Analtyics if needed */
  var googleAnalyticsMeasurementId = getConfig('googleAnalyticsMeasurementId');

  if (googleAnalyticsMeasurementId) {
    window.dataLayer = window.dataLayer || [];

    window.gtag = window.gtag || function gtag() {
      window.dataLayer.push(arguments);
    };

    window.gtag('js', new Date());
    window.gtag('config', googleAnalyticsMeasurementId);
  }

  window.analytics.addSourceMiddleware ? addMiddlewares() : window.analytics.ready(addMiddlewares);
}

export default analytics;
export { getAdobeVisitorData, getAdobeMCVisitorID } from './adobeRepository.js';
export { getUniversalId } from './universalId.js';